@@ -36,7 +36,7 @@ class Checkers:
         return self.path.relto(arg)
 
     def fnmatch(self, arg):
-        return FNMatcher(arg)(self.path)
+        return self.path.fnmatch(arg)
 
     def endswith(self, arg):
         return str(self.path).endswith(arg)
@@ -64,7 +64,10 @@ class Checkers:
                 else:
                     if bool(value) ^ bool(meth()) ^ invert:
                         return False
-            except (py.error.ENOENT, py.error.ENOTDIR):
+            except (py.error.ENOENT, py.error.ENOTDIR, py.error.EBUSY):
+                # EBUSY feels not entirely correct,
+                # but its kind of necessary since ENOMEDIUM
+                # is not accessible in python
                 for name in self._depend_on_existence:
                     if name in kw:
                         if kw.get(name):
@@ -75,7 +78,7 @@ class Checkers:
                             return False
         return True
 
-class NeverRaised(Exception): 
+class NeverRaised(Exception):
     pass
 
 class PathBase(object):
@@ -91,6 +94,11 @@ class PathBase(object):
         return self._getbyspec('basename')[0]
     basename = property(basename, None, None, basename.__doc__)
 
+    def dirname(self):
+        """ dirname part of path. """
+        return self._getbyspec('dirname')[0]
+    dirname = property(dirname, None, None, dirname.__doc__)
+
     def purebasename(self):
         """ pure base name of the path."""
         return self._getbyspec('purebasename')[0]
@@ -143,7 +151,7 @@ newline will be removed from the end of each line. """
     def move(self, target):
         """ move this path to target. """
         if target.relto(self):
-            raise py.error.EINVAL(target, 
+            raise py.error.EINVAL(target,
                 "cannot move path into a subdirectory of itself")
         try:
             self.rename(target)
@@ -156,27 +164,50 @@ newline will be removed from the end of each line. """
         return repr(str(self))
 
     def check(self, **kw):
-        """ check a path for existence, or query its properties
+        """ check a path for existence and properties.
 
-            without arguments, this returns True if the path exists (on the
-            filesystem), False if not
+            Without arguments, return True if the path exists, otherwise False.
 
-            with (keyword only) arguments, the object compares the value
-            of the argument with the value of a property with the same name
-            (if it has one, else it raises a TypeError)
+            valid checkers::
 
-            when for example the keyword argument 'ext' is '.py', this will
-            return True if self.ext == '.py', False otherwise
+                file=1    # is a file
+                file=0    # is not a file (may not even exist)
+                dir=1     # is a dir
+                link=1    # is a link
+                exists=1  # exists
+
+            You can specify multiple checker definitions, for example::
+
+                path.check(file=1, link=1)  # a link pointing to a file
         """
         if not kw:
             kw = {'exists' : 1}
         return self.Checkers(self)._evaluate(kw)
 
+    def fnmatch(self, pattern):
+        """return true if the basename/fullname matches the glob-'pattern'.
+
+        valid pattern characters::
+
+            *       matches everything
+            ?       matches any single character
+            [seq]   matches any character in seq
+            [!seq]  matches any char not in seq
+
+        If the pattern contains a path-separator then the full path
+        is used for pattern matching and a '*' is prepended to the
+        pattern.
+
+        if the pattern doesn't contain a path-separator the pattern
+        is only matched against the basename.
+        """
+        return FNMatcher(pattern)(self)
+
     def relto(self, relpath):
         """ return a string which is the relative part of the path
-        to the given 'relpath'. 
+        to the given 'relpath'.
         """
-        if not isinstance(relpath, (str, PathBase)): 
+        if not isinstance(relpath, (str, PathBase)):
             raise TypeError("%r: not a string or path object" %(relpath,))
         strrelpath = str(relpath)
         if strrelpath and strrelpath[-1] != self.sep:
@@ -187,17 +218,17 @@ newline will be removed from the end of each line. """
         if sys.platform == "win32" or getattr(os, '_name', None) == 'nt':
             if os.path.normcase(strself).startswith(
                os.path.normcase(strrelpath)):
-                return strself[len(strrelpath):]        
+                return strself[len(strrelpath):]
         elif strself.startswith(strrelpath):
             return strself[len(strrelpath):]
         return ""
 
-    def bestrelpath(self, dest): 
-        """ return a string which is a relative path from self 
-            (assumed to be a directory) to dest such that 
-            self.join(bestrelpath) == dest and if not such 
-            path can be determined return dest. 
-        """ 
+    def bestrelpath(self, dest):
+        """ return a string which is a relative path from self
+            (assumed to be a directory) to dest such that
+            self.join(bestrelpath) == dest and if not such
+            path can be determined return dest.
+        """
         try:
             if self == dest:
                 return os.curdir
@@ -212,12 +243,20 @@ newline will be removed from the end of each line. """
                 n = 0
             l = [os.pardir] * n
             if reldest:
-                l.append(reldest)     
+                l.append(reldest)
             target = dest.sep.join(l)
-            return target 
+            return target
         except AttributeError:
             return str(dest)
 
+    def exists(self):
+        return self.check()
+
+    def isdir(self):
+        return self.check(dir=1)
+
+    def isfile(self):
+        return self.check(file=1)
 
     def parts(self, reverse=False):
         """ return a root-first list of all ancestor directories
@@ -230,8 +269,8 @@ newline will be removed from the end of each line. """
             current = current.dirpath()
             if last == current:
                 break
-            l.insert(0, current)
-        if reverse:
+            l.append(current)
+        if not reverse:
             l.reverse()
         return l
 
@@ -259,11 +298,11 @@ newline will be removed from the end of each line. """
 
     def __lt__(self, other):
         try:
-            return self.strpath < other.strpath 
+            return self.strpath < other.strpath
         except AttributeError:
             return str(self) < str(other)
 
-    def visit(self, fil=None, rec=None, ignore=NeverRaised):
+    def visit(self, fil=None, rec=None, ignore=NeverRaised, bf=False, sort=False):
         """ yields all paths below the current one
 
             fil is a filter (glob pattern or callable), if not matching the
@@ -275,26 +314,14 @@ newline will be removed from the end of each line. """
 
             ignore is an Exception class that is ignoredwhen calling dirlist()
             on any of the paths (by default, all exceptions are reported)
+
+            bf if True will cause a breadthfirst search instead of the
+            default depthfirst. Default: False
+
+            sort if True will sort entries within each directory level.
         """
-        if isinstance(fil, str):
-            fil = FNMatcher(fil)
-        if rec: 
-            if isinstance(rec, str):
-                rec = fnmatch(fil)
-            elif not hasattr(rec, '__call__'):
-                rec = None
-        try:
-            entries = self.listdir()
-        except ignore:
-            return
-        dirs = [p for p in entries 
-                    if p.check(dir=1) and (rec is None or rec(p))]
-        for subdir in dirs:
-            for p in subdir.visit(fil=fil, rec=rec, ignore=ignore):
-                yield p
-        for p in entries:
-            if fil is None or fil(p):
-                yield p
+        for x in Visitor(fil, rec, ignore, bf, sort).gen(self):
+            yield x
 
     def _sortlist(self, res, sort):
         if sort:
@@ -307,30 +334,52 @@ newline will be removed from the end of each line. """
         """ return True if other refers to the same stat object as self. """
         return self.strpath == str(other)
 
+class Visitor:
+    def __init__(self, fil, rec, ignore, bf, sort):
+        if isinstance(fil, str):
+            fil = FNMatcher(fil)
+        if isinstance(rec, str):
+            self.rec = FNMatcher(rec)
+        elif not hasattr(rec, '__call__') and rec:
+            self.rec = lambda path: True
+        else:
+            self.rec = rec
+        self.fil = fil
+        self.ignore = ignore
+        self.breadthfirst = bf
+        self.optsort = sort and sorted or (lambda x: x)
+
+    def gen(self, path):
+        try:
+            entries = path.listdir()
+        except self.ignore:
+            return
+        rec = self.rec
+        dirs = self.optsort([p for p in entries
+                    if p.check(dir=1) and (rec is None or rec(p))])
+        if not self.breadthfirst:
+            for subdir in dirs:
+                for p in self.gen(subdir):
+                    yield p
+        for p in self.optsort(entries):
+            if self.fil is None or self.fil(p):
+                yield p
+        if self.breadthfirst:
+            for subdir in dirs:
+                for p in self.gen(subdir):
+                    yield p
+
 class FNMatcher:
     def __init__(self, pattern):
         self.pattern = pattern
+
     def __call__(self, path):
-        """return true if the basename/fullname matches the glob-'pattern'.
-
-        *       matches everything
-        ?       matches any single character
-        [seq]   matches any character in seq
-        [!seq]  matches any char not in seq
-
-        if the pattern contains a path-separator then the full path
-        is used for pattern matching and a '*' is prepended to the
-        pattern.
-
-        if the pattern doesn't contain a path-separator the pattern
-        is only matched against the basename.
-        """
         pattern = self.pattern
         if pattern.find(path.sep) == -1:
             name = path.basename
         else:
             name = str(path) # path.strpath # XXX svn?
-            pattern = '*' + path.sep + pattern
-        from fnmatch import fnmatch
-        return fnmatch(name, pattern)
+            if not os.path.isabs(pattern):
+                pattern = '*' + path.sep + pattern
+        return py.std.fnmatch.fnmatch(name, pattern)
 
