@@ -38,6 +38,9 @@ class CommonFSTests(object):
         p = path1.join('sample')
         p = p + 'dir'
         assert p.check()
+        assert p.exists()
+        assert p.isdir()
+        assert not p.isfile()
 
     def test_parts(self, path1):
         newpath = path1.join('sampledir', 'otherfile')
@@ -68,6 +71,10 @@ class CommonFSTests(object):
         assert newpath.check(basename='sampledir')
         assert newpath.basename, 'sampledir'
 
+    def test_dirname(self, path1):
+        newpath = path1.join('sampledir')
+        assert newpath.dirname == str(path1)
+
     def test_dirpath(self, path1):
         newpath = path1.join('sampledir')
         assert newpath.dirpath() == path1
@@ -89,16 +96,23 @@ class CommonFSTests(object):
     def test_exists(self, path1):
         assert path1.join("samplefile").check()
         assert path1.join("samplefile").check(exists=1)
+        assert path1.join("samplefile").exists()
+        assert path1.join("samplefile").isfile()
+        assert not path1.join("samplefile").isdir()
 
     def test_dir(self, path1):
         #print repr(path1.join("sampledir"))
         assert path1.join("sampledir").check(dir=1)
         assert path1.join('samplefile').check(notdir=1)
         assert not path1.join("samplefile").check(dir=1)
+        assert path1.join("samplefile").exists()
+        assert not path1.join("samplefile").isdir()
+        assert path1.join("samplefile").isfile()
 
     def test_fnmatch_file(self, path1):
         assert path1.join("samplefile").check(fnmatch='s*e')
-        assert path1.join("samplefile").check(notfnmatch='s*x')
+        assert path1.join("samplefile").fnmatch('s*e')
+        assert not path1.join("samplefile").fnmatch('s*x')
         assert not path1.join("samplefile").check(fnmatch='s*x')
 
     #def test_fnmatch_dir(self, path1):
@@ -283,6 +297,12 @@ class CommonFSTests(object):
     def test_relto_wrong_type(self, path1):
         py.test.raises(TypeError, "path1.relto(42)")
 
+    def test_load(self, path1):
+        p = path1.join('samplepickle')
+        obj = p.load()
+        assert type(obj) is dict
+        assert obj.get('answer',None) == 42
+
     def test_visit_filesonly(self, path1):
         l = []
         for i in path1.visit(lambda x: x.check(file=1)):
@@ -290,12 +310,6 @@ class CommonFSTests(object):
         assert not "sampledir" in l
         assert path1.sep.join(["sampledir", "otherfile"]) in l
 
-    def test_load(self, path1):
-        p = path1.join('samplepickle')
-        obj = p.load()
-        assert type(obj) is dict
-        assert obj.get('answer',None) == 42
-
     def test_visit_nodotfiles(self, path1):
         l = []
         for i in path1.visit(lambda x: x.check(dotfile=0)):
@@ -304,6 +318,28 @@ class CommonFSTests(object):
         assert path1.sep.join(["sampledir", "otherfile"]) in l
         assert not ".dotfile" in l
 
+    def test_visit_breadthfirst(self, path1):
+        l = []
+        for i in path1.visit(bf=True):
+            l.append(i.relto(path1))
+        for i, p in enumerate(l):
+            if path1.sep in p:
+                for j in range(i, len(l)):
+                    assert path1.sep in l[j]
+                break
+        else:
+            py.test.fail("huh")
+
+    def test_visit_sort(self, path1):
+        l = []
+        for i in path1.visit(bf=True, sort=True):
+            l.append(i.relto(path1))
+        for i, p in enumerate(l):
+            if path1.sep in p:
+                break
+        assert l[:i] == sorted(l[:i])
+        assert l[i:] == sorted(l[i:])
+
     def test_endswith(self, path1):
         def chk(p):
             return p.check(endswith="pickle")
