@@ -109,6 +109,11 @@ class TestLocalPath(common.CommonFSTests):
         finally:
             old.chdir()
 
+    @pytest.mark.xfail("sys.version_info < (2,6) and sys.platform == 'win32'")
+    def test_tilde_expansion(self):
+        p = py.path.local("~", expanduser=True)
+        assert p == os.path.expanduser("~")
+
     def test_eq_with_strings(self, path1):
         path1 = path1.join('sampledir')
         path2 = str(path1)
